from .Writer import Writer
from azure.storage.blob import BlockBlobService
import json
import pandas as pd


class ADLSWriter(Writer):
    """
        Write dataframe/json to specified blob storage location
    """
    def __init__(self, account_name, account_key, container_name, storage_name, dataflow_name):
        self.account_name = account_name
        self.account_key = account_key
        self.container_name = container_name
        self.storage_name = storage_name
        self.dataflow_name = dataflow_name
    
    def write_df(self, blob_location, dataframe, number_of_partition=1):
        """
            Write dataframe to specified blob storage location
        """
        #TODO partition the python df
        dataframe = dataframe.to_csv(index=False, header=False)
        block_blob_service = BlockBlobService(account_name=self.account_name, account_key=self.account_key)
        block_blob_service.create_blob_from_text(self.container_name + "/" + self.dataflow_name, blob_location, dataframe)
        blob_url = 'https://'+self.storage_name+'.dfs.core.windows.net/'+self.container_name+'/'+blob_location
        return blob_url

    def write_json(self, blob_location, json_dict):
        """
        write json to specified blob storage location
        """
        json_dict = json.dumps(json_dict)
        block_blob_service = BlockBlobService(
            account_name=self.account_name, account_key=self.account_key)
        block_blob_service.create_blob_from_text(self.container_name,self.dataflow_name+"/"+ blob_location, json_dict)
        blob_url = 'https://'+self.storage_name+'.dfs.core.windows.net/'+self.container_name+'/'+blob_location
        return blob_url


String = str
DateTimeOffset = str
Uri = str


def getattrIgnoreCase(obj, attr, default=None):
    for i in dir(obj):
        if i.lower() == attr.lower():
            return getattr(obj, attr, default)
    return default


dtype_converter = {
    'int': 'int64',
    'bigint': 'int64',
    'long': 'int64',
    'float':'decimal',
    'double': 'double',
    'decimal.Decimal': 'decimal',
    'string': 'string',
    'bool': 'boolean',
    'datetime': 'dateTime',
    'timestamp': 'dateTime'
}


from .Base import Base
from .JsonEnum import JsonEnum
from .utils import String
from .FileFormatSettings import FileFormatSettings

class CsvQuoteStyle(JsonEnum):
    Csv = "QuoteStyle.Csv"
    None_ = "QuoteStyle.None"


class CsvStyle(JsonEnum):
    QuoteAlways = "CsvStyle.QuoteAlways"
    QuoteAfterDelimiter = "CsvStyle.QuoteAfterDelimiter"


class CsvFormatSettings(FileFormatSettings):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("columnHeaders", bool, False),
            SchemaEntry("delimiter", String, ","),
            SchemaEntry("quoteStyle", CsvQuoteStyle, CsvQuoteStyle.Csv),
            SchemaEntry("csvStyle", CsvStyle, CsvStyle.QuoteAlways),
        ]
        super().__init__(self.schema)



from .Entity import Entity
from .SchemaEntry import SchemaEntry
from .utils import String
from .utils import DateTimeOffset


class ReferenceEntity(Entity):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("$type", String, "ReferenceEntity"),
            SchemaEntry("refreshTime", DateTimeOffset),
            SchemaEntry("source", String),
            SchemaEntry("modelId", String)
        ]
        super().__init__(self.schema)

    def toJson(self):
        entity = super().toJson()
        entity["$type"] = "ReferenceEntity"
        return entity
